	page    132,60
;----------------------------------------------------------------------------
;	BPSK.ASM
;----------------------------------------------------------------------------
; DSP interface program for the VE2IQ COHERENT program.
; Copyright (C) Johan Forrer, KC7WW, 1997. 
;----------------------------------------------------------------------------
; Obtain COHERENT from http://www.ietc.ca/home/bill/bbs.htm
;
; A couple of important points:
;
; 1) Change the EVM's ceramic resonator for a 4 MHz crystal.
; 2) Wire in a pair of RS232 to TTL convertors to change the
;    signal levels of RTS (used by COHERENT as the BPSK signal),
;    and DTR (used by COHERENT as the PTT switch). These level
;    shifters can be someting really simple. I connect the RS232
;    line in question to a 10K series resistor feeding into the
;    base of a 2N2222 transistor. The base of the transistor is 
;    then protected by a 1N914 diode (cathode to the transistor
;    base, anode to ground). The transistor emitter is grounded,
;    and the collector is then taken via a 1K resistor to +5V. 
;    I then routed a wire from the collector of each transistor
;    to PB8 and PB9 corresponding to RTS and DTR respectively.
; 3) Notice that the RED LED is your overload indicator - it has been
;    arranged that overload at the EVM corresponds to the CLIPPING
;    level for COHERENT.
; 4) The GREEN LED flashes at 1 second intervals. Check the accuracy
;    of your system against the second ticks on WWV. It should hold
;    time even for extended periods, however, if you can hold time for
;    even a minute, you are marginal, but it will work in a fashion.
; 5) The rest is up to you: study the COHERENT documentation for how
;    to use the program.
;
;    Good Luck!
;----------------------------------------------------------------------------
; Copyright (C) Johan Forrer, 1997, 
;----------------------------------------------------------------------------
; Johan B. Forrer, KC7WW
; 26553 Priceview Drive, Monroe, OR 97456, USA 
; <forrerj@peak.org>
; July, 1997.
;
; Disclaimer:
; This program is provided "as is" and is intended for educational
; purposes only. The author(s) cannot be held resonsible for any 
; mishap or accident resulting through the use of this software.
;  --Use at your own risk--
;
; This software is not to be used for any commercial application,
; military or government agencies without explicit written approval of the
; author(s).
;
; Acknowledgements:
; Pawel Jalocha, SP9VRC for many pieces of this code.
; Bill DeClarle, VE2IQ for help and assistance.
;
; MUCH-NEEDED IMPROVEMENTS:
; The sample rate changer introduces aliasing - fortunately the third order
; products are away from the 800 Hz passband, so that is why things
; works. If anyone adds a nice polyphase rate M/N rate changer, please keep
; me posted.
;----------------------------------------------------------------------------

	include 'sincod.asm'

;----------------------------------------------------------------------------
TONE_OUTPUT     EQU     HEADPHONE_EN+LINEOUT_EN+(0*LEFT_ATTN)+(0*RIGHT_ATTN)   
TONE_INPUT      EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)
OVR_CLR         EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)+$200000
;----------------------------------------------------------------------------
; Macros
;----------------------------------------------------------------------------
GrnLED  macro mode      ; Green LED clr/set/chg
	b\mode	#14,x:PBD
        endm

RedLED	macro	mode	; Red   LED clr/set/chg
	b\mode	#13,x:PBD
	endm

LeftPTT macro mode      ;PTT line: clr/set/chg
        b\mode #0,x:PBD
;       b\mode #4,x:PBD
        endm

RightPTT macro mode      ;PTT line: clr/set/chg
        b\mode #4,x:PBD
;       b\mode #0,x:PBD
        endm
;----------------------------------------------------------------------------

Carrier  	equ 	800.0
SampleFreq 	equ 	9600.0

PassFilterLen    equ 	43
PassFilterWidth  equ 	 500.0  ; Bandwidth required for modulation.
LowFreq = (Carrier-PassFilterWidth/2)/SampleFreq
UppFreq = (Carrier+PassFilterWidth/2)/SampleFreq


;----------------------------------------------------------------------------
LEFT 	equ	1
RIGHT	equ	2
usesig	equ	LEFT		; ***Use LEFT audio input***


;-----------------------------------------------------------------------------
	org     p:
main

; Initialize SC0, SC1 as SCI pins
    	bset    #0,x:PCC                 ; RXD
    	bset    #1,x:PCC                 ; TXD

; initialize port B
	movep	    #$0000,x:PBC	    ; port B as general purpose port
	movep	    #$60ff,x:PBDDR	    ; PB0-PB7,PB13 and PB14 as outputs
	movep	    #$0000,x:PBD


	movep	#$1,x:TCSR		; timer mode 0, GPIO
;;	movep	#>1249,x:TCR		; 36Mhz/2/7200/2 - 1  double rate for tests
	movep	#>2499,x:TCR		; 36Mhz/2/7200/2 - 1  timer factor

	move	#>3600,a
	move	a,x:tcount		; clear tcount


; Initialize SCI port
; Use internal clocks, 115.2 bps
;	
	movep	#>$0302,x:SCR		; 1 start, 1 stop, 8 data (10 bits) 

	movep	#>$0004,x:SCCR		; CD=(36 MHz/bps/64) - 1 = 4 (115.2)
					; (112500 or -2.34%)
;;	movep	#>$003A,x:SCCR		; CD=(36 MHz/bps/64) - 1 = 58 (9.6)
;;					; (9533.9 or -0.69%)


	movep	#$02301b,x:IPR		; IRQA/IRQB/SSI are level 3 interrupts.
					; and are LEVEL sensitive.
					; Timer is level 1 (SCI are polled).
	movep   #$fb00,x:CRB            ; Enable the SSI interrupts.

	movep	#>$3,x:TCSR		; Enable Timer interrupts	

	
        ori #%00000100,omr      	; enable the sine ROM table
                                	; this is for the FreqSynth routine

;
; Program headphone,lineout enabled, speaker off, attn=0
; Does this need to done on evey cycle or once in the beginning?
;
	move    #TONE_OUTPUT,y0         ; headphones, line out, 
					; mute spkr, no attn.
	move    y0,x:TX_BUFF_BASE+2
	move    #TONE_INPUT,y0          ; no input gain, monitor mute
	move    y0,x:TX_BUFF_BASE+3


;---------------------------------------------------------------------------
; Initialize buffer pointer
;---------------------------------------------------------------------------
	move	#Buffer,r1	; data_in_ptr = input buffer pointer
	move	r1,x:data_in_ptr

	move	#OutBuf,r1	; out_ptr = output buffer pointer
	move	r1,x:out_ptr
;---------------------------------------------------------------------------
; Main Loop
loop_1 
	jset    #2,x:SSISR,*            ; Wait for frame sync to pass.
	jclr    #2,x:SSISR,*            ; Wait for frame sync.

	move    x:RX_BUFF_BASE,a        ; Load the left channel input.
	move    x:RX_BUFF_BASE+1,b      ; Load the right channel input.
	
;----------------------------------------------------------------------------
; Read status of switches/RTS/DTR
;----------------------------------------------------------------------------
	movep	x:PBD,a
	move	a1,x:switches

;----------------------------------------------------------------------------
; Read input signal.
; Note: Input may be overdriven, which case results will saturate.
; Detect overload condition from Slot 7, bit 5 (RX_BUFF_BASE+3 = $20)
; or when read left justified into a 24-bit 20xxxx.
; The bit is reset by writing zero.
;----------------------------------------------------------------------------
	move	a1,x:LEFT_sig		; save A/D data
	move	b1,x:RIGHT_sig

; Test for CODEC overload condition
	move    x:<RX_BUFF_BASE+3,a
	move	#>$200000,x0
	and	x0,a
	jeq	sig_good
	RedLED	set
	move    #TONE_INPUT,y0		; clear overload bit
	move    y0,x:<TX_BUFF_BASE+3
	jmp	process
sig_good
	RedLED	clr
	move    #OVR_CLR,y0		; enable overload detection
	move    y0,x:<TX_BUFF_BASE+3
process

;----------------------------------------------------------------------------
; Play back audio for testing purposes
;----------------------------------------------------------------------------
;;	move	x:Bpfout,a		; Retrieve data
;;	rep #8
;;	asl	a
;;	move    a1,x:TX_BUFF_BASE        ; Put value in left channel tx.
;;	move    a1,x:TX_BUFF_BASE+1      ; Put value in right channel tx.  
	
;-----------------------------------------------------------------------------
; Output tone to left and right channel
;-----------------------------------------------------------------------------
	move	x:Toneout,a
	asr	a
	move    a1,x:TX_BUFF_BASE
	move    a1,x:TX_BUFF_BASE+1

;----------------------------------------------------------------------------
; Prepare data pointers to receive sampled data
;----------------------------------------------------------------------------
	move	x:data_in_ptr,r1
        move    #PassFilterLen-1,m1     ; mod(ntaps)

 IF (usesig==LEFT) 			; Select which input to use
	move	x:LEFT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r1)+		; save in circular buffer
 ENDIF

 IF (usesig==RIGHT)
	move	x:RIGHT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r1)+		; save in circular buffer
 ENDIF

	move	r1,x:data_in_ptr	; save updated pointer

;---------------------------------------------------------------------------
; Perform input bandpass filtering
; Sample rate = 9600 SPS
; On completion, pointers is same as in beginning, accum a contains result
;---------------------------------------------------------------------------
        move    #PassFilter,r4	; R4 point to filter coefficients
        move    #>-1,m4     	; linear addressing
	nop

        clr     a    x:(r1)+,x0  y:(r4)+,y0     ; priming registers
        .loop	#PassFilterLen-1		; allow interrupts
	mac     x0,y0,a  x:(r1)+,x0  y:(r4)+,y0
	.endl
        macr    x0,y0,a		; final rounding
	move	a,x:Bpfout	; save it


;----------------------------------------------------------------------------
; Read and extract RTS status for doing phase shifting
;----------------------------------------------------------------------------
	move	x:switches,a
	move	#>$100,x0
	and	x0,a1
	jne	passthru
	move	x:cosine,a
	neg	a
	jmp	output

passthru
	move	x:cosine,a

;----------------------------------------------------------------------------
; Prepare to bandpass filter output tone.
; We use the same identical filter as the input bandpass.
;----------------------------------------------------------------------------
output

	move	x:out_ptr,r1
        move    #PassFilterLen-1,m1     ; mod(ntaps)
	nop
	move	a1,x:(r1)+
	move	r1,x:out_ptr

        move    #PassFilter,r4	; R4 point to filter coefficients
        move    #>-1,m4     	; linear addressing
	nop

        clr     a    x:(r1)+,x0  y:(r4)+,y0     ; priming registers
        .loop	#PassFilterLen-1		; allow interrupts
	mac     x0,y0,a  x:(r1)+,x0  y:(r4)+,y0
	.endl
        macr    x0,y0,a		; final rounding

	move	a,x:Toneout		; save output tone

;----------------------------------------------------------------------------
; Update carrier phase
;----------------------------------------------------------------------------
        move 	X:<CarPhase,x0  	; get phase
        move 	X:<CarFreq,a    	; advance the phase
        add 	x0,a
        move 	a1,X:<CarPhase
        jsr 	<FreqSynth              ; Compute sine/cosines
					; modifies a,b,x,y,r1,m1,n1
	move	a1,x:cosine
	move	b1,x:sine


;----------------------------------------------------------------------------
; Read and extract DTR status for PTT operation
;----------------------------------------------------------------------------
	move	x:switches,a
	move	#>$200,x0
	and	x0,a1
	jne	Release

 IF (usesig==LEFT) 			; Select Left/Right PTT
	LeftPTT set
 else
	RightPTT set
 endif
	jmp	loop_exit

Release
 IF (usesig==LEFT)
	LeftPTT clr
 else
	RightPTT clr
 endif

loop_exit
	jmp     loop_1                  ; Loop back.
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
; Routine computes a cosine/sine pair using the sine ROM
; with a second order (linear+quadrature) approximation 
; between table points.
; 	x,y,r1,m1,n1 are modified.
; 	Maximum error is about 0.7E-6
; 	Execution time 4+64+4 clock cycles including "jsr <IQ_lev2" and "rts"
;-----------------------------------------------------------------------------
PI      equ     3.14159265358979323846
FreqSynth                             	; x0 = angle ( -1 = -PI, +1 = +PI)
        move 	#>$80,x1   		; shift out 8 most significant bits
        mpy 	x0,x1,a  #>$FF,x0
        move 	x0,m1
        and 	x0,a     #>$100,x0
        or 	x0,a     #<$40,n1
        move 	a1,r1      		; put the 8 most significant bits 
					; into r1 with offset = $100
        move 	a0,y0      		; save the remaining bits in y0
        jclr 	#23,y0,SinTable_lev2
        move 	(r1)+
SinTable_lev2
        move 	Y:(r1+n1),x0       	; x0 = coarse cosine
        move 	Y:(r1),x1          	; x1 = coarse sine
        mpyr 	x1,y0,a  #PI/256.0,y1
        tfr 	x0,a  a,x1
        macr 	-x1,y1,a           	;a = fine cosine
        mpyr 	x0,y0,b  Y:(r1),x1
        tfr 	x1,b  b,x1
        macr 	x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr 	y0,y0,a  a,x0
        move 	a,y0
        mpyr 	y0,y1,a
        tfr 	x0,a  a,y1
        macr 	-x0,y1,a  b,x1     	;a = super fine cosine
        macr 	-x1,y1,b           	;b = super fine sine
        rts


;*************************************************************************************
; Timer ISR
TIMER:
	move    a0,x:save_a0
	move    a1,x:save_a1
	move    a2,x:save_a2
	move    x0,x:save_x0

	move	x:tcount,a
	move	#>1,x0
	sub	x0,a
	jne	done		
	GrnLED	chg
	move	#>3600,a
done
	move	a1,x:tcount

;----------------------------------------------------------------------------
; Form 7 bits+sign to simulate S-D board
; Data = (Current - Previous), scaled to signed 8-bit value
;----------------------------------------------------------------------------
	move	x:Bpfout,a
	move	x:Previous,x0
	move	a,x:Previous
	sub	x0,a
	rep #7
	asr	a
	move	a,x:temp		; save it

	abs	a			; value >127   
	move	#>127,x0
	sub	x0,a
	jgt	error			; flag error if it does
	move	x:temp,a   		; otherwise its OK
	jmp	outit		
error
	move	x:temp,a
	move	#>$7F,x0
	or	x0,a	
outit
	movep	a1,x:STX

	move    x:save_a0,a0
	move    x:save_a1,a1
	move    x:save_a2,a2
	move    x:save_x0,x0
	rti

;*************************************************************************************


;-----------------------------------------------------------------------------
       org     x:      
LEFT_sig        ds      1       ;storage for Left Signal
RIGHT_sig       ds      1       ;storage for Left Signal


CarPhase  	dc 	0
CarFreq   	dc 	2.0*Carrier/SampleFreq
sine		ds	1	; Sine/Cosine results
cosine		ds	1

tcount		ds	1	; 7200 Hz downcounter
Previous	ds	1
data_in_ptr	ds	1	; buffer pointer
out_ptr		ds	1	; buffer pointer
Bpfout		ds	1	; Raw filtered outputs
Toneout		ds	1	; Carrier tone output
switches	ds	1	; PB inputs
temp		ds	1	; temporary

save_r0		ds	1 	; Temporary register storage
save_m0		ds	1
save_a0		ds	1
save_a1		ds	1
save_a2		ds	1
save_x0		ds	1

Buffer		dsm	PassFilterLen	; storage for input  data
OutBuf		dsm	PassFilterLen	; storage for output data

;-----------------------------------------------------------------------------

       org     y:      
PassFilter ds PassFilterLen ; anti-alias input filter

	org Y:PassFilter

Pi      equ     3.14159265358979323846
time = -@cvf(PassFilterLen/2)+0.5
count   set 0
	dup PassFilterLen
Window = 0.75*@cos((time/@cvf(PassFilterLen))*Pi)+0.25*@cos(3*(time/@cvf(PassFilterLen))*Pi)
Filter = (@sin(2.0*Pi*time*UppFreq)-@sin(2.0*Pi*time*LowFreq))/(Pi*time)
time = time+1.0
	dc Window*Filter
count   set count+1
	endm
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
; Reserve Y:0100-01FF for sine/cosine lookup table
;-----------------------------------------------------------------------------
       org     y:$0100
lookup		ds	256
;-----------------------------------------------------------------------------

   end
